#!/bin/sh

#----------------------------------------------------------------------#
# Functions
#----------------------------------------------------------------------#
log() {
		echo $1
		echo `date +"%b %d %T: "`$1 >> /cathexis/logs/upgrade.log
}

logWithoutEcho() {
		echo `date +"%b %d %T: "`$1 >> /cathexis/logs/upgrade.log
}

exitFail () {
		log "Error: $1"
		rm -fr $TMP_DIR
		exit 1
}

exitInterrupt () {
		exitFail "Upgrade interrupted"
}


#----------------------------------------------------------------------#
# Initial checks
#----------------------------------------------------------------------#
if [ $# -ne 1 ]
then
		echo "Usage: "`basename $0`" pack.upg"
		exit 1
fi

PACK_FILENAME=`readlink -f $1`
if [ $? -ne 0 ]
then
		echo "Pack $1 doesn't exist"
		exit 1
fi


logWithoutEcho "-------------------------------------------------------------"
log "Pack: $PACK_FILENAME"

TMP_DIR=`mktemp -dt -p /space/tmp`
if [ $? -ne 0 ]
then
		log "Failed to create working directory"
		exit 1
fi
log "Working dir: $TMP_DIR"

trap exitInterrupt INT


#----------------------------------------------------------------------#
# And run
#----------------------------------------------------------------------#
cd $TMP_DIR

log "Unpacking..."
tar xzpf $PACK_FILENAME
if [ $? -ne 0 ]
then
		exitFail "Failed to extract pack"
fi

#-- Load upgrade configuration ----------------------
if [ -f upgrade.conf ]
then
		log "Loading upgrade configuration"
		. ./upgrade.conf
fi


#-- Log upgrade decription --------------------------
if [ -f upgrade.description ]
then
		UPGRADE_DESCRIPTION=`cat upgrade.description`
else
		UPGRADE_DESCRIPTION="unknown"
fi
log "Upgrade description: $UPGRADE_DESCRIPTION"


case "$UPGRADE_DESCRIPTION" in
  *5042*)
		exitFail "Cannot downgrade to 5042 - operation not allowed"
		;;
  *5043a*)
		exitFail "Cannot downgrade to 5043a - operation not allowed"
		;;
  *5043b1*)
		exitFail "Cannot downgrade to 5043b1 - operation not allowed"
		;;
esac

#-- Upgrade kernel ----------------------------------
if [ -f kernel.tgz ]
then
		log "Extracting kernel"
		tar -x -z -p -f kernel.tgz
		if [ $? -ne 0 ]
		then
				exitFail "Failed to extract kernel"
		fi

		if [ ! -f uImage ]
		then
				exitFail "Extracted kernel does not contain uImage"
		fi
		
		log "Erasing existing kernel"
		flash_eraseall /dev/mtd4 | awk '{gsub("\r","\n ->"); print $0 }'
		if [ $? -ne 0 ]
		then
				log "flash_eraseall failed"
		fi

		log "Writing new kernel"
		nandwrite -p /dev/mtd4 uImage | awk '{ print " -> "$0 }'
		if [ $? -ne 0 ]
		then
				exitFail "nandwrite failed"
		else
				log "Kernel updated"
		fi
fi


#-- Extract utils -----------------------------------
if [ -f utils.tgz ]
then
		log "Extracting utils"
		tar -x -z -p -f utils.tgz
		if [ $? -ne 0 ]
		then
				exitFail "Failed to extract utils"
		fi
fi


#-- Run pre-install script --------------------------
if [ -x pre_install.sh ]
then
		log "Running pre-install script"
		./pre_install.sh
		if [ $? -ne 0 ]
		then
				exitFail "Pre-install script failed"
		fi
fi


#-- Extract upgrade ---------------------------------
if [ -f upgrade.tgz ]
then
		log "Extracting upgrade"

		tar -C / -x -z -p -f upgrade.tgz
		if [ $? -ne 0 ]
		then
				exitFail "Failed to extract upgrade"
		fi
fi


#-- Run post-install script -------------------------
if [ -x post_install.sh ]
then
		log "Running post-install script"
		./post_install.sh
		if [ $? -ne 0 ]
		then
				exitFail "Post-install script failed"
		fi
fi

#-- Cleanup -----------------------------------------
log "Cleaning temp dir $TMP_DIR"
rm -fr $TMP_DIR
log "Upgrade complete"

REBOOT=${REBOOT:-1}

if [ $REBOOT -eq 1 ]
then
		log "Reboot..."
		reboot
else
		log "Not rebooting"
fi

exit 0
